from typing import List

class Interval:
    """
    Represents a range of values.
    """

    def __new__(cls, start: int, end: int) -> Interval:
        """
        Create a new Interval object.

        :param start: The start of the interval.
        :param end: The end of the interval.
        """

    @property
    def start(self) -> int:
        """
        The start of the interval.
        """

    @property
    def end(self) -> int:
        """
        The end of the interval.
        """

    def __repr__(self) -> str: ...

class AIList:
    """
    The augmented interval list (AILIST) object.

    This object will compute region overlaps very efficiently.
    """

    def __new__(
        cls, intervals: List[Interval], minimum_coverage_length: int = None
    ) -> AIList:
        """
        Create a new AIList object.

        :param intervals: A list of intervals.
        :param minimum_coverage_length: The minimum length of the coverage.
        """

    def query(self, interval: Interval) -> List[Interval]:
        """
        Query the AIList object for overlapping intervals.

        :param interval: The interval to query.
        """



from typing import List

class Region:
    chr: str
    start: int
    end: int

    rest: str

    def __repr__(self): ...
    def __str__(self): ...
    def __len__(self) -> int:
        """
        Size of the region
        """
        ...

class RegionSet:
    regions: List[Region]
    header: str
    path: str

    def __init__(self, path: str) -> "RegionSet":
        """
        :param path: path to the bed file
        """
        ...

    @property
    def identifier(self) -> str:
        """
        Get RegionSet identifier

        :return: bed digest/identifier
        """
        ...

    @property
    def path(self) -> str:
        """
        Get RegionSet identifier

        :return: bed file path
        """
        ...

    @property
    def header(self) -> str:
        """
        Header of the bed file
        """
        ...

    def to_bed(self, path: str) -> None:
        """
        Save RegionSet as bed file

        :param path: path to the bed file that has to be saved
        """
        ...

    def to_bed_gz(self, path: str) -> None:
        """
        Save RegionSet as bed.gz file

        :param path: path to the bed file that has to be saved
        """
        ...

    def to_bigbed(self, out_path: str, chrom_size: str) -> None:
        """
        Save RegionSet as bigBed file

        :param out_path: path bigbed file path
        :param chrom_size: path to the chrom sizes file
        """
        ...

    def sort(self) -> None:
        """
        Sort the regions
        """
        ...

    def __len__(self) -> int:
        """
        Size of the regionset
        """
        ...

    def __iter__(self): ...
    def __getitem__(self, indx: int): ...
    def __repr__(self): ...
    def __str__(self): ...



from typing import List

def write_tokens_to_gtok(filename: str, tokens: List[str]):
    """
    Write tokens to a GTOK file.

    :param filename: The filename of the GTOK file.
    :param tokens: The tokens to write.
    """
    pass

def read_tokens_from_gtok(filename: str) -> List[str]:
    """
    Read tokens from a GTOK file.

    :param filename: The filename of the GTOK file.
    """
    pass
